import base64
import hashlib
from io import BytesIO

import pyvips
from celery import Celery
from celery.utils.log import get_task_logger
from minio import Minio
from minio.error import S3Error
from sqlalchemy import select, update
from sqlalchemy.orm import Session

from foxnouns.db import User
from foxnouns.db.sync import engine
from foxnouns.settings import MINIO, REDIS_URL

app = Celery("tasks", broker=REDIS_URL)

logger = get_task_logger(__name__)

minio = Minio(
    MINIO["ENDPOINT"],
    access_key=MINIO["ACCESS_KEY"],
    secret_key=MINIO["SECRET_KEY"],
    secure=MINIO["SECURE"],
)
bucket = MINIO["BUCKET"]


def convert_avatar(uri: str) -> bytes:
    if not uri.startswith("data:image/"):
        raise ValueError("Not a data URI")

    content_type, encoded = uri.removeprefix("data:").split("base64,", 1)
    logger.info(f"{content_type=}")

    img = pyvips.Image.thumbnail_buffer(
        base64.b64decode(encoded),
        512,
        height=512,
        size=pyvips.Size.BOTH,
        crop=pyvips.Interesting.CENTRE,
    )
    return img.write_to_buffer(".webp", Q=95)


@app.task
def process_user_avatar(user_id: int, avatar: str):
    with Session(engine) as conn:
        user = conn.scalar(select(User).where(User.id == user_id))
    if not user:
        raise ValueError("process_user_avatar was passed the ID of a nonexistent user")

    img = convert_avatar(avatar)
    hash = hashlib.new("sha256", data=img).hexdigest()
    old_hash = user.avatar

    minio.put_object(
        bucket,
        f"users/{user_id}/avatars/{hash}.webp",
        BytesIO(img),
        len(img),
        "image/webp",
    )

    with Session(engine) as conn:
        conn.execute(update(User).values(avatar=hash).where(User.id == user_id))
        conn.commit()

    if not old_hash:
        return

    try:
        minio.remove_object(bucket, f"users/{user_id}/avatars/{old_hash}.webp")
    except S3Error as exc:
        logger.debug(f"{exc.code=}")
        # TODO: filter out 404 errors
        raise exc


@app.task
def delete_user_avatar(user_id: int):
    with Session(engine) as conn:
        user = conn.scalar(select(User).where(User.id == user_id))
    if not user:
        raise ValueError("delete_user_avatar was passed the ID of a nonexistent user")
    if not user.avatar:
        logger.info(
            "delete_user_avatar was called for a user with a null avatar (%d)", user_id
        )
        return

    try:
        minio.remove_object(bucket, f"users/{user_id}/avatars/{user.avatar}.webp")

        with Session(engine) as conn:
            conn.execute(update(User).values(avatar=None).where(User.id == user_id))
            conn.commit()

    except S3Error as exc:
        logger.debug(f"{exc.code=}")
        # TODO: filter out 404 errors
        raise exc
